searchData={"items":[{"type":"module","title":"PhoenixTest","doc":"PhoenixTest provides a unified way of writing feature tests -- regardless of\nwhether you're testing LiveView pages or static pages.\n\nIt also handles navigation between LiveView and static pages seamlessly. So, you\ndon't have to worry about what type of page you're visiting. Just write the\ntests from the user's perspective.\n\nThus, you can test a flow going from static to LiveView pages and back without\nhaving to worry about the underlying implementation.\n\nThis is a sample flow:\n\n```elixir\ntest \"admin can create a user\", %{conn: conn} do\n  conn\n  |> visit(\"/\")\n  |> click_link(\"Users\")\n  |> fill_in(\"Name\", with: \"Aragorn\")\n  |> choose(\"Ranger\")\n  |> assert_has(\".user\", text: \"Aragorn\")\nend\n```\n\nNote that PhoenixTest does _not_ handle JavaScript. If you're looking for\nsomething that supports JavaScript, take a look at\n[Wallaby](https://hexdocs.pm/wallaby/readme.html)\nor the external [PhoenixTest.Playwright](https://hexdocs.pm/phoenix_test_playwright) driver.","ref":"PhoenixTest.html"},{"type":"module","title":"Setup - PhoenixTest","doc":"PhoenixTest requires Phoenix `1.7+` and LiveView `0.20+`. It may work with\nearlier versions, but I have not tested that.","ref":"PhoenixTest.html#module-setup"},{"type":"module","title":"Installation - PhoenixTest","doc":"Add `phoenix_test` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:phoenix_test, \"~> 0.7.0\", only: :test, runtime: false}\n  ]\nend\n```","ref":"PhoenixTest.html#module-installation"},{"type":"module","title":"Configuration - PhoenixTest","doc":"In `config/test.exs` specify the endpoint to be used for routing requests:\n\n```elixir\nconfig :phoenix_test, :endpoint, MyAppWeb.Endpoint\n```","ref":"PhoenixTest.html#module-configuration"},{"type":"module","title":"Getting `PhoenixTest` helpers - PhoenixTest","doc":"`PhoenixTest` helpers can be included via `import PhoenixTest`.\n\nBut since each test needs a `conn` struct to get started, you'll likely want\nto set up a few things before that.\n\nThere are two ways to do that.","ref":"PhoenixTest.html#module-getting-phoenixtest-helpers"},{"type":"module","title":"With `ConnCase` - PhoenixTest","doc":"If you plan to use `ConnCase` solely for `PhoenixTest`, then you can import\nthe helpers there:\n\n```elixir\nusing do\n  quote do\n    # importing other things for ConnCase\n\n    import PhoenixTest\n\n    # doing other setup for ConnCase\n  end\nend\n```","ref":"PhoenixTest.html#module-with-conncase"},{"type":"module","title":"Adding a `FeatureCase` - PhoenixTest","doc":"If you want to create your own `FeatureCase` helper module like `ConnCase`,\nyou can copy the code below which can be `use`d from your tests (replace\n`MyApp` with your app's name):\n\n```elixir\ndefmodule MyAppWeb.FeatureCase do\n  use ExUnit.CaseTemplate\n\n  using do\n    quote do\n      use MyAppWeb, :verified_routes\n\n      import MyAppWeb.FeatureCase\n\n      import PhoenixTest\n    end\n  end\n\n  setup tags do\n    pid = Ecto.Adapters.SQL.Sandbox.start_owner!(MyApp.Repo, shared: not tags[:async])\n    on_exit(fn -> Ecto.Adapters.SQL.Sandbox.stop_owner(pid) end)\n\n    {:ok, conn: Phoenix.ConnTest.build_conn()}\n  end\nend\n```\n\nNote that we assume your Phoenix project is using Ecto and its phenomenal\n`SQL.Sandbox`. If it doesn't, feel free to remove the `SQL.Sandbox` code\nabove.","ref":"PhoenixTest.html#module-adding-a-featurecase"},{"type":"module","title":"Usage - PhoenixTest","doc":"Now that we have all the setup out of the way, we can create tests like\nthis:\n\n```elixir\n# test/my_app_web/features/admin_can_create_user_test.exs\n\ndefmodule MyAppWeb.AdminCanCreateUserTest do\n  use MyAppWeb.FeatureCase, async: true\n\n  test \"admin can create user\", %{conn: conn} do\n    conn\n    |> visit(\"/\")\n    |> click_link(\"Users\")\n    |> fill_in(\"Name\", with: \"Aragorn\")\n    |> fill_in(\"Email\", with: \"aragorn@dunedain.com\")\n    |> click_button(\"Create\")\n    |> assert_has(\".user\", text: \"Aragorn\")\n  end\nend\n```","ref":"PhoenixTest.html#module-usage"},{"type":"module","title":"Filling out forms - PhoenixTest","doc":"We can fill out forms by targetting their inputs, selects, etc. by label:\n\n```elixir\ntest \"admin can create user\", %{conn: conn} do\n  conn\n  |> visit(\"/\")\n  |> click_link(\"Users\")\n  |> fill_in(\"Name\", with: \"Aragorn\")\n  |> select(\"Elessar\", from: \"Aliases\")\n  |> choose(\"Human\") # <- choose a radio option\n  |> check(\"Ranger\") # <- check a checkbox\n  |> click_button(\"Create\")\n  |> assert_has(\".user\", text: \"Aragorn\")\nend\n```\n\nFor more info, see `fill_in/3`, `select/3`, `choose/3`, `check/2`,\n`uncheck/2`.","ref":"PhoenixTest.html#module-filling-out-forms"},{"type":"module","title":"Submitting forms without clicking a button - PhoenixTest","doc":"Once we've filled out a form, you can click a button with\n`click_button/2` to submit the form. But sometimes you want to emulate what\nwould happen by just pressing  .\n\nFor that case, you can use `submit/1` to submit the form you just filled\nout.\n\n```elixir\nsession\n|> fill_in(\"Name\", with: \"Aragorn\")\n|> check(\"Ranger\")\n|> submit()\n```\n\nFor more info, see `submit/1`.","ref":"PhoenixTest.html#module-submitting-forms-without-clicking-a-button"},{"type":"module","title":"Targeting which form to fill out - PhoenixTest","doc":"If you find yourself in a situation where you have multiple forms with the\nsame labels (even when those labels point to different inputs), then you\nmight have to scope your form-filling.\n\nTo do that, you can scope all of the form helpers using `within/3`:\n\n```elixir\nsession\n|> within(\"#user-form\", fn session ->\n  session\n  |> fill_in(\"Name\", with: \"Aragorn\")\n  |> check(\"Ranger\")\n  |> click_button(\"Create\")\nend)\n```\n\nFor more info, see `within/3`.","ref":"PhoenixTest.html#module-targeting-which-form-to-fill-out"},{"type":"function","title":"PhoenixTest.assert_has/2","doc":"Assert helper to ensure an element with given CSS selector is present.\n\nIt'll raise an error if no elements are found, but it will _not_ raise if more\nthan one matching element is found.\n\nIf you want to specify the content of the element, use `assert_has/3`.","ref":"PhoenixTest.html#assert_has/2"},{"type":"function","title":"Examples - PhoenixTest.assert_has/2","doc":"```elixir\n# assert there's an h1\nassert_has(session, \"h1\")\n\n# assert there's an element with ID \"user\"\nassert_has(session, \"#user\")\n```","ref":"PhoenixTest.html#assert_has/2-examples"},{"type":"function","title":"PhoenixTest.assert_has/3","doc":"Assertion helper to ensure an element with given CSS selector and contents is\nfound\n\nIt'll raise an error if no elements are found, but it will _not_ raise if more\nthan one matching element is found (unless `:count` option is used).\n\nYou can assert an element's contents using `:text` or assert a field's value\nusing `:value` (with an optional `:label`).\n\nNOTE that you cannot specify both `:text` and `:value` as options.","ref":"PhoenixTest.html#assert_has/3"},{"type":"function","title":"Options - PhoenixTest.assert_has/3","doc":"- `text`: the text contents to look for.\n\n- `value`: the element's value to look for.\n\n- `label`: the label associated to the form field with `value`\n\n- `exact`: by default `assert_has/3` will perform a substring match (e.g. `a\n=~ b`). That makes it easier to assert text within HTML elements that also\ncontain other HTML elements. But sometimes we want to assert the exact text is\npresent. For that, use `exact: true`. Note: when used with `:value`, the\nexactness applies to the label's text, not the input's value. (defaults to\n`false`)\n\n- `count`: the number of items you expect to match CSS selector (and `text` or\n`value` if provided)\n\n- `at`: (integer) position of the element to be asserted against\n\n- `timeout`: currently only works with LiveViews. If you pass a positive\n`timeout`, PhoenixTest will wait for any async operations (i.e. LiveView's\n`assign_async` and `start_async`) and handle redirects that happen as result\nof a `handle_async` or `handle_info`. (defaults to `0`)","ref":"PhoenixTest.html#assert_has/3-options"},{"type":"function","title":"Examples - PhoenixTest.assert_has/3","doc":"```elixir\n# assert there's an element with ID \"user\" and text \"Aragorn\"\nassert_has(session, \"#user\", text: \"Aragorn\")\n  # ^ succeeds if text found is \"Aragorn\" or \"Aragorn, Son of Arathorn\"\n\n# assert there's an element with ID \"user\" and text \"Aragorn\"\nassert_has(session, \"#user\", text: \"Aragorn\", exact: true)\n  # ^ succeeds only if text found is \"Aragorn\". Fails if finds \"Aragorn, Son of Arathorn\"\n\n# assert there's an input with value \"Frodo\"\nassert_has(session, \"input\", value: \"Frodo\")\n\n# assert there's an input with value \"Frodo\" labeled by \"Hobbit\"\nassert_has(session, \"input\", value: \"Frodo\", label: \"Hobbit\")\n\n# assert there are two elements with class \"posts\"\nassert_has(session, \".posts\", count: 2)\n\n# assert there are two elements with class \"posts\" and text \"Hello\"\nassert_has(session, \".posts\", text: \"Hello\", count: 2)\n\n# assert the second element in the list of \".posts\" has text \"Hello\"\nassert_has(session, \".posts\", at: 2, text: \"Hello\")\n\n# assert the h1 has text \"Hello\" in the next 100ms (perhaps as a result of an\n# async assign)\nassert_has(session, \"h1\", text: \"Hello\", timeout: 100)\n```","ref":"PhoenixTest.html#assert_has/3-examples"},{"type":"function","title":"PhoenixTest.assert_path/2","doc":"Assert helper to verify current request path. Takes an optional `query_params`\nmap.\n\n>","ref":"PhoenixTest.html#assert_path/2"},{"type":"function","title":"Note on Live Patch Implementation {: .info} - PhoenixTest.assert_path/2","doc":">\n> Capturing the current path in live patches relies on message passing and\ncould, therefore, be subject to intermittent failures. Please open an issue if\nyou see intermittent failures when using `assert_path` with live patches so we\ncan improve the implementation.","ref":"PhoenixTest.html#assert_path/2-note-on-live-patch-implementation-info"},{"type":"function","title":"Examples - PhoenixTest.assert_path/2","doc":"```elixir\n# assert we're at /users\nconn\n|> visit(\"/users\")\n|> assert_path(\"/users\")\n\n# assert we're at /users?name=frodo\nconn\n|> visit(\"/users\")\n|> assert_path(\"/users\", query_params: %{name: \"frodo\"})\n\n# assert we're at a path with a wildcard. The wildcard represents a single part of the path.\nconn\n|> visit(\"/users\")\n|> click_link(\"Any User\")\n|> assert_path(\"/users/*/profile\")\n```","ref":"PhoenixTest.html#assert_path/2-examples"},{"type":"function","title":"PhoenixTest.assert_path/3","doc":"Same as `assert_path/2` but takes an optional `query_params` map.","ref":"PhoenixTest.html#assert_path/3"},{"type":"function","title":"PhoenixTest.check/3","doc":"Check a checkbox.\n\nTo uncheck a checkbox, see `uncheck/3`.","ref":"PhoenixTest.html#check/3"},{"type":"function","title":"Options - PhoenixTest.check/3","doc":"- `exact`: whether to match label text exactly. (Defaults to `true`)","ref":"PhoenixTest.html#check/3-options"},{"type":"function","title":"Inside a form - PhoenixTest.check/3","doc":"If the form is a LiveView form, and if the form has a `phx-change` attribute\ndefined, `check/3` will trigger the `phx-change` event.\n\nThis can be followed by a `click_button/3` or `submit/1` to submit the form.","ref":"PhoenixTest.html#check/3-inside-a-form"},{"type":"function","title":"Example - PhoenixTest.check/3","doc":"Given we have a form that contains this:\n\n```html\n Admin \n \n \n```\n\nWe can check the \"Admin\" option:\n\n```elixir\nsession\n|> check(\"Admin\")\n```","ref":"PhoenixTest.html#check/3-example"},{"type":"function","title":"Outside of a form - PhoenixTest.check/3","doc":"If the checkbox exists outside of a form, `check/3` will trigger the\n`phx-click` event.","ref":"PhoenixTest.html#check/3-outside-of-a-form"},{"type":"function","title":"Example - PhoenixTest.check/3","doc":"```html\n Admin \n \n```\n\nWe can check the \"Admin\" option:\n\n```elixir\nsession\n|> check(\"Admin\")\n```\n\nAnd that will send a `\"toggle-admin\"` event with the input's `value` as the\npayload.","ref":"PhoenixTest.html#check/3-example"},{"type":"function","title":"Complex labels - PhoenixTest.check/3","doc":"If we have a complex label, you can use `exact: false` to target part of the\nlabel.","ref":"PhoenixTest.html#check/3-complex-labels"},{"type":"function","title":"Example - PhoenixTest.check/3","doc":"Given we have a form that contains this:\n\n```html\n Admin  *  \n \n \n```\n\nWe can check the \"Admin\" option:\n\n```elixir\nsession\n|> check(\"Admin\", exact: false)\n```","ref":"PhoenixTest.html#check/3-example"},{"type":"function","title":"PhoenixTest.check/4","doc":"Like `check/3` but allows you to specify the checkbox's CSS selector.\n\nHelpful in cases when you have multiple checkboxes with the same label on the\nsame form.\n\nFor more on checking boxes, see `check/3`. To uncheck a checkbox, see\n`uncheck/3` and `uncheck/4`.","ref":"PhoenixTest.html#check/4"},{"type":"function","title":"PhoenixTest.choose/3","doc":"Choose a radio button option.","ref":"PhoenixTest.html#choose/3"},{"type":"function","title":"Options - PhoenixTest.choose/3","doc":"- `exact`: whether to match label text exactly. (Defaults to `true`)","ref":"PhoenixTest.html#choose/3-options"},{"type":"function","title":"Inside a form - PhoenixTest.choose/3","doc":"If the form is a LiveView form, and if the form has a `phx-change` attribute\ndefined, `choose/3` will trigger the `phx-change` event.\n\nThis can be followed by a `click_button/3` or `submit/1` to submit the form.\n\nIf the radio button exists outside of a form, `choose/3` will trigger the\n`phx-click` event.","ref":"PhoenixTest.html#choose/3-inside-a-form"},{"type":"function","title":"Example - PhoenixTest.choose/3","doc":"Given we have a form that contains this:\n\n```html\n \n Email \n\n \n Phone \n```\n\nWe can choose to be contacted by email:\n\n```elixir\nsession\n|> choose(\"Email\")\n```","ref":"PhoenixTest.html#choose/3-example"},{"type":"function","title":"Outside of a form - PhoenixTest.choose/3","doc":"If the checkbox exists outside of a form, `choose/3` will trigger the\n`phx-click` event.","ref":"PhoenixTest.html#choose/3-outside-of-a-form"},{"type":"function","title":"Example - PhoenixTest.choose/3","doc":"```html\n \n Email \n```\n\nWe can choose to be contacted by email:\n\n```elixir\nsession\n|> choose(\"Email\")\n```\n\nAnd we'll get a `\"select-contact\"` event with the input's value in the payload.","ref":"PhoenixTest.html#choose/3-example"},{"type":"function","title":"Complex labels - PhoenixTest.choose/3","doc":"If we have a complex label, you can use `exact: false` to target part of the\nlabel.","ref":"PhoenixTest.html#choose/3-complex-labels"},{"type":"function","title":"Example - PhoenixTest.choose/3","doc":"Given we have a form that contains this:\n\n```html\n \n Email  *  \n```\n\nWe can choose to be contacted by email:\n\n```elixir\nsession\n|> choose(\"Email\", exact: false)\n```","ref":"PhoenixTest.html#choose/3-example"},{"type":"function","title":"PhoenixTest.choose/4","doc":"Like `choose/3` but you can specify an input's selector (in addition to the\nlabel).\n\nHelpful for cases when you have multiple radio buttons with the same label.","ref":"PhoenixTest.html#choose/4"},{"type":"function","title":"Example - PhoenixTest.choose/4","doc":"Consider a form containig the following:\n\n```heex\n \n   Do you like Elixir: \n\n   \n     \n     Yes \n   \n   \n     \n     No \n   \n \n\n \n   Do you like Erlang: \n\n   \n     \n     Yes \n   \n   \n     \n     No \n   \n \n```\n\nSince all radio buttons have the labels \"Yes\" or \"No\", you can target a\nspecific radio button like so:\n\n```elixir\nsession\n|> choose(\"#elixir-yes\", \"Yes\")\n```","ref":"PhoenixTest.html#choose/4-example"},{"type":"function","title":"PhoenixTest.click_button/2","doc":"Perfoms action defined by button with given text (using a substring match).\nThe action is based on attributes present.\n\nThis can be used in a number of ways.","ref":"PhoenixTest.html#click_button/2"},{"type":"function","title":"Button with `phx-click` - PhoenixTest.click_button/2","doc":"If the button has a `phx-click` on it, it'll send the event to the LiveView.","ref":"PhoenixTest.html#click_button/2-button-with-phx-click"},{"type":"function","title":"Example - PhoenixTest.click_button/2","doc":"```html\n Save \n```\n\n```elixir\nsession\n|> click_button(\"Save\") # <- will send \"save\" event to LiveView\n```","ref":"PhoenixTest.html#click_button/2-example"},{"type":"function","title":"Button relying on Phoenix.HTML.js - PhoenixTest.click_button/2","doc":"If the button acts as a form via Phoenix.HTML's `data-method`, `data-to`, and\n`data-csrf`, this will emulate Phoenix.HTML.js and submit the form via data\nattributes.\n\nBut note that this _doesn't guarantee_ the JavaScript that handles form\nsubmissions via `data` attributes is loaded. The test emulates the behavior\nbut you must make sure the JavaScript is loaded.\n\nFor more on that, see https://hexdocs.pm/phoenix_html/Phoenix.HTML.html#module-javascript-library","ref":"PhoenixTest.html#click_button/2-button-relying-on-phoenix-html-js"},{"type":"function","title":"Example - PhoenixTest.click_button/2","doc":"```html\n Delete \n```\n\n```elixir\nsession\n|> click_button(\"Delete\") # <- will submit form like Phoenix.HTML.js does\n```","ref":"PhoenixTest.html#click_button/2-example"},{"type":"function","title":"Combined with `fill_in/3`, `select/3`, etc. - PhoenixTest.click_button/2","doc":"This function can be preceded by filling out a form.","ref":"PhoenixTest.html#click_button/2-combined-with-fill_in-3-select-3-etc"},{"type":"function","title":"Example - PhoenixTest.click_button/2","doc":"```elixir\nsession\n|> fill_in(\"Name\", name: \"Aragorn\")\n|> check(\"Human\")\n|> click_button(\"Create\")\n```","ref":"PhoenixTest.html#click_button/2-example"},{"type":"function","title":"Submitting default data - PhoenixTest.click_button/2","doc":"By default, using `click_button/2` will submit the form it's part of (so long\nas it has a `phx-click`, `data-*` attrs, or an `action`).\n\nIt will also include any hidden inputs and default data (e.g. inputs with a\n`value` set and the button's `name` and `value` if present).","ref":"PhoenixTest.html#click_button/2-submitting-default-data"},{"type":"function","title":"Example - PhoenixTest.click_button/2","doc":"```html\n \n   \n   Complete \n \n```\n\n```elixir\nsession\n|> click_button(\"Complete\")\n# ^ includes `%{\"admin\" => \"true\", \"complete\" => \"true\"}` in payload\n```","ref":"PhoenixTest.html#click_button/2-example"},{"type":"function","title":"Single-button forms - PhoenixTest.click_button/2","doc":"`click_button/2` is smart enough to use a hidden input's value with\n`name=_method` as the method to send (e.g. when we want to send `delete`,\n`put`, or `patch`)\n\nThat means, it is helpful to submit single-button forms.","ref":"PhoenixTest.html#click_button/2-single-button-forms"},{"type":"function","title":"Example - PhoenixTest.click_button/2","doc":"```html\n \n   \n   Delete \n \n```\n\n```elixir\nsession\n|> click_button(\"Delete\") # <- Triggers full form delete.\n```","ref":"PhoenixTest.html#click_button/2-example"},{"type":"function","title":"Clicking elements that behave as buttons - PhoenixTest.click_button/2","doc":"Even though we can technically add an `onClick` or `phx-click` handler on any\nHTML element, the HTML spec encourages that clickable elements be links or\nbuttons.\n\nBut in some cases, people want to have other elements act as buttons even if\nthey don't use a ` ` tag.\n\nTo that end, `click_button/2` also supports clicking these elements:\n\n- elements with the \"button\" role (e.g. ` I'm clickable `)\n- `input` elements with types: \"button\", \"image\", \"reset\", or \"submit\"\n\nFor more on the \"button\" role, see [MDN's button role](https://developer.mozilla.org/en-US/docs/Web/Accessibility/ARIA/Reference/Roles/button_role)","ref":"PhoenixTest.html#click_button/2-clicking-elements-that-behave-as-buttons"},{"type":"function","title":"PhoenixTest.click_button/3","doc":"Performs action defined by button with CSS selector and text.\n\nSee `click_button/2` for more details.","ref":"PhoenixTest.html#click_button/3"},{"type":"function","title":"PhoenixTest.click_link/2","doc":"Clicks a link with given text (using a substring match) and performs the\naction.\n\nHere's how it handles different types of `a` tags:\n\n- With `href`: follows it to the next page\n- With `phx-click`: it'll send the event to the appropriate LiveView\n- With live redirect: it'll follow the live navigation to the next LiveView\n- With live patch: it'll patch the current LiveView","ref":"PhoenixTest.html#click_link/2"},{"type":"function","title":"Examples - PhoenixTest.click_link/2","doc":"```heex\n<.link href=\"/page/2\">Page 2</.link>\n<.link phx-click=\"next-page\">Next Page</.link>\n<.link navigate=\"next-liveview\">Next LiveView</.link>\n<.link patch=\"page/details\">Page Details</.link>\n```\n\n```elixir\nsession\n|> click_link(\"Page 2\") # <- follows to next page\n\nsession\n|> click_link(\"Next Page\") # <- sends \"next-page\" event to LiveView\n\nsession\n|> click_link(\"Next LiveView\") # <- follows to next LiveView\n\nsession\n|> click_link(\"Page Details\") # <- applies live patch\n```","ref":"PhoenixTest.html#click_link/2-examples"},{"type":"function","title":"Submitting forms - PhoenixTest.click_link/2","doc":"Phoenix allows for submitting forms on links via Phoenix.HTML's `data-method`,\n`data-to`, and `data-csrf`.\n\nWe can use `click_link` to emulate Phoenix.HTML.js and submit the\nform via data attributes.\n\nBut note that this _doesn't guarantee_ the JavaScript that handles form\nsubmissions via `data` attributes is loaded. The test emulates the behavior\nbut you must make sure the JavaScript is loaded.\n\nFor more on that, see https://hexdocs.pm/phoenix_html/Phoenix.HTML.html#module-javascript-library","ref":"PhoenixTest.html#click_link/2-submitting-forms"},{"type":"function","title":"Example - PhoenixTest.click_link/2","doc":"```html\n \n  Delete\n \n```\n\n```elixir\nsession\n|> click_link(\"Delete\") # <- will submit form like Phoenix.HTML.js does\n```","ref":"PhoenixTest.html#click_link/2-example"},{"type":"function","title":"PhoenixTest.click_link/3","doc":"Clicks a link with given CSS selector and text and performs the action.\nselector to target the link.\n\nSee `click_link/2` for more details.","ref":"PhoenixTest.html#click_link/3"},{"type":"function","title":"PhoenixTest.fill_in/3","doc":"Fills text inputs and textareas, targetting the elements by their labels.\n\nThis can be followed by a `click_button/3` or `submit/1` to submit the form.\n\nIf the form is a LiveView form, and if the form has a `phx-change` attribute\ndefined, `fill_in/3` will trigger the `phx-change` event.","ref":"PhoenixTest.html#fill_in/3"},{"type":"function","title":"Options - PhoenixTest.fill_in/3","doc":"- `with` (required): the text to fill in.\n\n- `exact`: whether to match label text exactly. (Defaults to `true`)","ref":"PhoenixTest.html#fill_in/3-options"},{"type":"function","title":"Examples - PhoenixTest.fill_in/3","doc":"Given we have a form that contains this:\n\n```html\n Name \n \n```\n\nor this:\n\n```html\n \n  Name\n   \n \n```\n\nWe can fill in the `name` field:\n\n```elixir\nsession\n|> fill_in(\"Name\", with: \"Aragorn\")\n```","ref":"PhoenixTest.html#fill_in/3-examples"},{"type":"function","title":"Complex labels - PhoenixTest.fill_in/3","doc":"If we have a complex label, you can use `exact: false` to target part of the\nlabel.","ref":"PhoenixTest.html#fill_in/3-complex-labels"},{"type":"function","title":"Example - PhoenixTest.fill_in/3","doc":"Given the following:\n\n```html\n Name  *  \n \n```\n\nWe can fill in the `name` field:\n\n```elixir\nsession\n|> fill_in(\"Name\", with: \"Aragorn\", exact: false)\n```","ref":"PhoenixTest.html#fill_in/3-example"},{"type":"function","title":"Labels and Accessibility - PhoenixTest.fill_in/3","doc":"This function requires a label to target the input element. This is by design, as labels are\ncrucial for accessibility. Labels provide both visual and programmatic association with inputs,\nallowing screen readers to announce the input's purpose when focused.\n\nIf you need to hide a label visually while maintaining accessibility, use a screen-reader-only\nclass (like Tailwind's `sr-only`) rather than removing the label. For example:\n\n```html\n Search \n \n```","ref":"PhoenixTest.html#fill_in/3-labels-and-accessibility"},{"type":"function","title":"PhoenixTest.fill_in/4","doc":"Like `fill_in/3` but you can specify an input's selector (in addition to the\nlabel).\n\nHelpful for cases when you have multiple fields with the same label.","ref":"PhoenixTest.html#fill_in/4"},{"type":"function","title":"Example - PhoenixTest.fill_in/4","doc":"Consider a form containig the following:\n\n```html\n \n   \n     First Name \n     \n   \n \n\n \n   \n     First Name \n     \n   \n \n```\n\nSince each new contact gets the same \"First Name\" label, you can target a\nspecific input like so:\n\n```elixir\nsession\n|> fill_in(\"#contact_1_first_name\", \"First Name\", with: \"Aragorn\")\n```","ref":"PhoenixTest.html#fill_in/4-example"},{"type":"function","title":"PhoenixTest.open_browser/1","doc":"Open the default browser to display current HTML of `session`.","ref":"PhoenixTest.html#open_browser/1"},{"type":"function","title":"Examples - PhoenixTest.open_browser/1","doc":"```elixir\nsession\n|> visit(\"/\")\n|> fill_in(\"Name\", with: \"Aragorn\")\n|> open_browser()\n|> submit()\n```","ref":"PhoenixTest.html#open_browser/1-examples"},{"type":"function","title":"PhoenixTest.refute_has/2","doc":"Opposite of `assert_has/2` helper. Verifies that element with\ngiven CSS selector is _not_ present.\n\nIt'll raise an error if any elements that match selector are found.\n\nIf you want to specify the content of the element, use `refute_has/3`.","ref":"PhoenixTest.html#refute_has/2"},{"type":"function","title":"Example - PhoenixTest.refute_has/2","doc":"```elixir\n# refute there's an h1\nrefute_has(session, \"h1\")\n\n# refute there's an element with ID \"user\"\nrefute_has(session, \"#user\")\n```","ref":"PhoenixTest.html#refute_has/2-example"},{"type":"function","title":"PhoenixTest.refute_has/3","doc":"Opposite of `assert_has/3` helper. Verifies that element with\ngiven CSS selector and content is _not_ present.\n\nIt'll raise an error if any elements that match selector and options.\n\nNOTE that you cannot specify both `:text` and `:value` as options.","ref":"PhoenixTest.html#refute_has/3"},{"type":"function","title":"Options - PhoenixTest.refute_has/3","doc":"- `text`: the text filter to look for.\n\n- `value`: the element's value to look for.\n\n- `label`: the label associated to the form field with `value`\n\n- `exact`: by default `refute_has/3` will perform a substring match (e.g. `a\n=~ b`). That makes it easier to refute text within HTML elements that also\ncontain other HTML elements. But sometimes we want to refute the exact text is\nabsent. For that, use `exact: true`. Note: when used with `:value`, the\nexactness applies to the label's text, not the input's value. (defaults to\n`false`)\n\n- `count`: the number of items you're expecting _should not_ match the CSS\nselector (and `text` or `value` if provided)\n\n- `at`: (integer) position of the element to be refuted against\n\n- `timeout`: experimental option that only works with LiveViews. If you pass a\npositive `timeout`, PhoenixTest will wait for any async operations (i.e.\nLiveView's `assign_async` and `start_async`) and handle redirects that happen\nas result of a `handle_async` or `handle_info`. (defaults to `0`)","ref":"PhoenixTest.html#refute_has/3-options"},{"type":"function","title":"Examples - PhoenixTest.refute_has/3","doc":"```elixir\n# refute there's an element with ID \"user\" and text \"Aragorn\"\nrefute_has(session, \"#user\", text: \"Aragorn\")\n\n# refute there's an element with ID \"user\" and exact text \"Aragorn\"\nrefute_has(session, \"#user\", text: \"Aragorn\", exact: true)\n\n# refute there's an input with value \"Frodo\"\nrefute_has(session, \"input\", value: \"Frodo\")\n\n# refute there's an input with value \"Frodo\" and label \"Hobbit\"\nrefute_has(session, \"input\", value: \"Frodo\", label: \"Hobbit\")\n\n# refute there are two elements with class \"posts\" (less or more will not raise)\nrefute_has(session, \".posts\", count: 2)\n\n# refute there are two elements with class \"posts\" and text \"Hello\"\nrefute_has(session, \".posts\", text: \"Hello\", count: 2)\n\n# refute the second element with class \"posts\" has text \"Hello\"\nrefute_has(session, \".posts\", at: 2, text: \"Hello\")\n\n# refute the \"h1\" has text \"Hello\" in the next 100ms (perhaps as a result of\n# an async assign)\nrefute_has(session, \"h1\", text: \"Hello\", timeout: 100)\n```","ref":"PhoenixTest.html#refute_has/3-examples"},{"type":"function","title":"PhoenixTest.refute_path/2","doc":"Verifies current request path is NOT the one provided. Takes an optional\n`query_params` map for more specificity.\n\n>","ref":"PhoenixTest.html#refute_path/2"},{"type":"function","title":"Note on Live Patch Implementation {: .info} - PhoenixTest.refute_path/2","doc":">\n> Capturing the current path in live patches relies on message passing and\ncould, therefore, be subject to intermittent failures. Please open an issue if\nyou see intermittent failures when using `refute_path` with live patches so we\ncan improve the implementation.","ref":"PhoenixTest.html#refute_path/2-note-on-live-patch-implementation-info"},{"type":"function","title":"Examples - PhoenixTest.refute_path/2","doc":"```elixir\n# refute we're at /posts\nconn\n|> visit(\"/users\")\n|> refute_path(\"/posts\")\n\n# refute we're at /users?name=frodo\nconn\n|> visit(\"/users?name=aragorn\")\n|> refute_path(\"/users\", query_params: %{name: \"frodo\"})\n```","ref":"PhoenixTest.html#refute_path/2-examples"},{"type":"function","title":"PhoenixTest.refute_path/3","doc":"Same as `refute_path/2` but takes an optional `query_params` for more specific\nrefutation.","ref":"PhoenixTest.html#refute_path/3"},{"type":"function","title":"PhoenixTest.select/3","doc":"Selects an option from a select dropdown.","ref":"PhoenixTest.html#select/3"},{"type":"function","title":"Options - PhoenixTest.select/3","doc":"- `option` (required): the text of the option to select.\n\n- `from` (deprecated): the label of the select dropdown. Label is now a\npositional argument and use the `:option` option to pass the option text.\n\n- `exact`: whether to match label text exactly. (Defaults to `true`)\n\n- `exact_option`: whether to match the option's text exactly. (Defaults to `true`)","ref":"PhoenixTest.html#select/3-options"},{"type":"function","title":"Inside a form - PhoenixTest.select/3","doc":"If the form is a LiveView form, and if the form has a `phx-change` attribute\ndefined, `select/3` will trigger the `phx-change` event.\n\nThis can be followed by a `click_button/3` or `submit/1` to submit the form.","ref":"PhoenixTest.html#select/3-inside-a-form"},{"type":"function","title":"Example - PhoenixTest.select/3","doc":"Given we have a form that contains this:\n\n```html\n \n   Race \n   \n     Human \n     Elf \n     Dwarf \n     Orc \n   \n \n```\n\nWe can select an option:\n\n```elixir\nsession\n|> select(\"Race\", option: \"Human\")\n```","ref":"PhoenixTest.html#select/3-example"},{"type":"function","title":"Outside a form - PhoenixTest.select/3","doc":"If the select dropdown exists outside of a form, `select/3` will trigger the\n`phx-click` event associated to the option being selected (note that all\noptions must have a `phx-click` in that case).","ref":"PhoenixTest.html#select/3-outside-a-form"},{"type":"function","title":"Examples - PhoenixTest.select/3","doc":"Given we have a form that contains this:\n\n```html\n Race \n \n   Human \n   Elf \n   Dwarf \n   Orc \n \n```\n\nWe can select an option:\n\n```elixir\nsession\n|> select(\"Race\", option: \"Human\")\n```\n\nAnd we'll get an event `\"select-race\"` with the payload `%{\"value\" =>\n\"human\"}`.","ref":"PhoenixTest.html#select/3-examples"},{"type":"function","title":"Complex labels - PhoenixTest.select/3","doc":"If we have a complex label, you can use `exact: false` to target part of the\nlabel.","ref":"PhoenixTest.html#select/3-complex-labels"},{"type":"function","title":"Example - PhoenixTest.select/3","doc":"Given we have a form that contains this:\n\n```html\n Race  *  \n \n   Human \n   Elf \n   Dwarf \n   Orc \n \n```\n\nWe can select an option:\n\n```elixir\nsession\n|> select(\"Race\", exact: false, option: \"Human\")\n```","ref":"PhoenixTest.html#select/3-example"},{"type":"function","title":"PhoenixTest.select/4","doc":"Like `select/3` but you can specify a select's CSS selector (in addition to\nthe label).\n\nHelpful when you have multiple selects with the same label.\n\nFor more on selecting options, see `select/3`.","ref":"PhoenixTest.html#select/4"},{"type":"function","title":"PhoenixTest.submit/1","doc":"Helper to submit a pre-filled form without clicking a button (see `fill_in/3`,\n`select/3`, `choose/3`, etc. for how to fill a form.)\n\nForms are typically submitted by clicking buttons. But sometimes we want to\nemulate what happens when we submit a form hitting  . That's what this\nhelper does.\n\nIf the form is a LiveView form, and if the form has a `phx-submit` attribute\ndefined, `submit/1` will trigger the `phx-submit` event. Otherwise, it'll\nsubmit the form regularly.\n\nIf the form has a submit button with a `name` and `value`, `submit/1` will\nalso include that data in the payload.","ref":"PhoenixTest.html#submit/1"},{"type":"function","title":"Example - PhoenixTest.submit/1","doc":"```elixir\nsession\n|> fill_in(\"Name\", with: \"Aragorn\")\n|> select(\"Human\", from: \"Race\")\n|> choose(\"Email\")\n|> submit()\n```","ref":"PhoenixTest.html#submit/1-example"},{"type":"function","title":"PhoenixTest.uncheck/3","doc":"Uncheck a checkbox.\n\nTo check a checkbox, see `check/3`.","ref":"PhoenixTest.html#uncheck/3"},{"type":"function","title":"Options - PhoenixTest.uncheck/3","doc":"- `exact`: whether to match label text exactly. (Defaults to `true`)","ref":"PhoenixTest.html#uncheck/3-options"},{"type":"function","title":"Inside a form - PhoenixTest.uncheck/3","doc":"If the form is a LiveView form, and if the form has a `phx-change` attribute\ndefined, `uncheck/3` will trigger the `phx-change` event.\n\nThis can be followed by a `click_button/3` or `submit/1` to submit the form.","ref":"PhoenixTest.html#uncheck/3-inside-a-form"},{"type":"function","title":"Example - PhoenixTest.uncheck/3","doc":"Given we have a form that contains this:\n\n```html\n Admin \n \n \n```\n\nWe can uncheck the \"Admin\" option:\n\n```elixir\nsession\n|> uncheck(\"Admin\")\n```\n\nNote that unchecking a checkbox in HTML doesn't actually send any data to the\nserver. That's why we have to have a hidden input with the default value (in\nthe example above: `admin=\"off\"`).","ref":"PhoenixTest.html#uncheck/3-example"},{"type":"function","title":"Outside of a form - PhoenixTest.uncheck/3","doc":"If the checkbox exists outside of a form, `uncheck/3` will trigger the\n`phx-click` event and send an empty (`%{}`) payload.","ref":"PhoenixTest.html#uncheck/3-outside-of-a-form"},{"type":"function","title":"Example - PhoenixTest.uncheck/3","doc":"```html\n Admin \n \n```\n\nWe can uncheck the \"Admin\" option:\n\n```elixir\nsession\n|> uncheck(\"Admin\")\n```\n\nAnd that will send a `\"toggle-admin\"` event with an empty map `%{}` as a\npayload.","ref":"PhoenixTest.html#uncheck/3-example"},{"type":"function","title":"Complex labels - PhoenixTest.uncheck/3","doc":"If we have a complex label, you can use `exact: false` to target part of the\nlabel.","ref":"PhoenixTest.html#uncheck/3-complex-labels"},{"type":"function","title":"Example - PhoenixTest.uncheck/3","doc":"Given we have a form that contains this:\n\n```html\n Admin  *  \n \n \n```\n\nWe can uncheck the \"Admin\" option:\n\n```elixir\nsession\n|> uncheck(\"Admin\", exact: false)\n```","ref":"PhoenixTest.html#uncheck/3-example"},{"type":"function","title":"PhoenixTest.uncheck/4","doc":"Like `uncheck/3` but allows you to specify the checkbox's CSS selector.\n\nHelpful when you have multiple checkboxes with the same label. In those cases,\nyou might need to specify the selector of the labeled element.\n\nNote that in those cases, the selector should point to the checkbox that is\nvisible, not to the hidden input. For more, see `uncheck/2`.\n\nFor more on unchecking boxes, see `uncheck/3`. To check a checkbox, see\n`check/3` and `check/4`.","ref":"PhoenixTest.html#uncheck/4"},{"type":"function","title":"PhoenixTest.unwrap/2","doc":"Escape hatch to give users access to underlying \"native\" data structure.\n\nOnce the unwrapped actions are performed, PhoenixTest will handle redirects\n(if any).\n\n- In LiveView tests, `unwrap/2` will pass the `view` that comes from\nPhoenix.LiveViewTest `live/2`. Your action _must_ return the result of a\n`render_*` LiveViewTest action.\n\n- In non-LiveView tests, `unwrap/2` will pass the `conn` struct. And your\naction _must_ return a `conn` struct.","ref":"PhoenixTest.html#unwrap/2"},{"type":"function","title":"Examples - PhoenixTest.unwrap/2","doc":"```elixir\n# in a LiveView\nsession\n|> unwrap(fn view ->\n  view\n  |> LiveViewTest.element(\"#hook\")\n  |> LiveViewTest.render_hook(:hook_event, %{name: \"Legolas\"})\nend)\n```\n\n```elixir\n# in a non-LiveView\nsession\n|> unwrap(fn conn ->\n  conn\n  |> Phoenix.ConnTest.recycle()\nend)\n```","ref":"PhoenixTest.html#unwrap/2-examples"},{"type":"function","title":"PhoenixTest.upload/4","doc":"Upload a file.\n\nIf the form is a LiveView form, this will perform a live file upload and trigger the associated `phx-change` event.\n\nThis can be followed by a `click_button/3` or `submit/1` to submit the form.\n\n> LiveView Note: If `allow_upload/3` has a `:progress` callback that issues a navigate or redirect,\n> it will be used instead of any navigate, redirect or patch from the upload's `phx-change` event.","ref":"PhoenixTest.html#upload/4"},{"type":"function","title":"Options - PhoenixTest.upload/4","doc":"- `exact`: whether to match the entire label. (Defaults to `true`)","ref":"PhoenixTest.html#upload/4-options"},{"type":"function","title":"Examples - PhoenixTest.upload/4","doc":"Given we have a form that contains this:\n\n```html\n Avatar \n \n```\n\nWe can upload a file:\n\n```elixir\nsession\n|> upload(\"Avatar\", \"/path/to/file\")\n```","ref":"PhoenixTest.html#upload/4-examples"},{"type":"function","title":"Complex labels - PhoenixTest.upload/4","doc":"If we have a complex label, you can use `exact: false` to target part of the\nlabel.","ref":"PhoenixTest.html#upload/4-complex-labels"},{"type":"function","title":"Example - PhoenixTest.upload/4","doc":"Given the following:\n\n```html\n Avatar  *  \n \n```\n\nWe can upload a file:\n\n```elixir\nsession\n|> upload(\"Avatar\", \"/path/to/file\", exact: false)\n```","ref":"PhoenixTest.html#upload/4-example"},{"type":"function","title":"PhoenixTest.upload/5","doc":"Like `upload/4` but you can specify an input's selector (in addition to the\nlabel).\n\nHelpful in cases when you have uploads with the same label on the same form.\n\nFor more, see `upload/4`.","ref":"PhoenixTest.html#upload/5"},{"type":"function","title":"PhoenixTest.visit/2","doc":"Entrypoint to create a session.\n\n`visit/2` takes a `Plug.Conn` struct and the path to visit.\n\nIt returns a `session` which the rest of the `PhoenixTest` functions can use.\n\nNote that `visit/2` is smart enough to know if the page you're visiting is a\nLiveView or a static view. You don't need to worry about which type of page\nyou're visiting.","ref":"PhoenixTest.html#visit/2"},{"type":"function","title":"PhoenixTest.within/3","doc":"Helpers to scope filling out form within a given selector. Use this if you\nhave more than one form on a page with similar labels.","ref":"PhoenixTest.html#within/3"},{"type":"function","title":"Examples - PhoenixTest.within/3","doc":"Given we have some HTML like this:\n\n```html\n \n   Name \n   \n\n   \n   Admin \n   \n \n\n# and assume another form with \"Name\" and \"Admin\" labels\n```\n\nWe can fill the form like this:\n\n```elixir\nsession\n|> within(\"#user-form\", fn session ->\n  session\n  |> fill_in(\"Name\", with: \"Aragorn\")\n  |> check(\"Admin\")\nend)\n```","ref":"PhoenixTest.html#within/3-examples"},{"type":"extras","title":"Changelog","doc":"Changelog\n=========\n\nNoteworthy changes are included here. For a full version of changes, see git\nhistory.\n\nTo see dates a version was published, see the [hex package\npage](https://hex.pm/packages/phoenix_test)","ref":"changelog.html"},{"type":"extras","title":"0.7.0 - Changelog","doc":"","ref":"changelog.html#0-7-0"},{"type":"extras","title":"Potentially breaking change - Changelog","doc":"- If you test uploads. PhoenixTest will now automatically trigger `phx-change`\n  on `upload`. If you were previously triggering `phx-change` after your\n  `upload`, you might now get duplicate `phx-change` events. See PR [#162] or\n  commit [8edd7b4] for more information.","ref":"changelog.html#potentially-breaking-change"},{"type":"extras","title":"New - Changelog","doc":"- Adds: asserting by value and label (#204). Commit [515242d]\n- Adds asserting & refuting by `:value` (#199). Commit [6271e81]","ref":"changelog.html#new"},{"type":"extras","title":"Improvements - Changelog","doc":"- Allow `JS.patch` in `phx-click` bindings. Commit [6012517]\n- Handles LiveView forms that remove elements (#202). Commit [6f610f2]\n- Allow empty maps to be passed into `assert_path` as `query_params` (#196).\n  Commit [04ca498]\n- `assert_path` handles query params with [] (#195). Commit [ac52f8b]\n- Include simple `phx-*` attributes in selectors (#193). Commit [a327ccf]","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Do not raise non-existent path if URL is external (#206). Commits [891733f]\n- Pass the original filter text from `click_button/2` when calling\n  `click_button/3` (#222). Commit [dc17378]\n- Fix: Find select nested in label (#210). Commit [450ad7d]\n- return empty value for button if name is present and no value is found (#220).\n  Commit [45ee501]\n- Use Jason instead of native JSON to support < 1.18. Commit [632a5c6]\n- Fix: pass view to assert_redirect. Commit [2b73714]\n- Fix: Handle prefilled values for type=email etc. Commit [ec4440d]\n- Test all \"simple-value\" input defaults. Commit [45b62ca]\n- Check local path's endpoint at runtime to fix warning. Commit [1dbf12e]\n\n[#162]: https://github.com/germsvel/phoenix_test/pull/162\n[dc17378]: https://github.com/germsvel/phoenix_test/commit/dc17378\n[1dbf12e]: https://github.com/germsvel/phoenix_test/commit/1dbf12e\n[f9d1bb9]: https://github.com/germsvel/phoenix_test/commit/f9d1bb9\n[45ee501]: https://github.com/germsvel/phoenix_test/commit/45ee501\n[450ad7d]: https://github.com/germsvel/phoenix_test/commit/450ad7d\n[632a5c6]: https://github.com/germsvel/phoenix_test/commit/632a5c6\n[6012517]: https://github.com/germsvel/phoenix_test/commit/6012517\n[d99c572]: https://github.com/germsvel/phoenix_test/commit/d99c572\n[7a9b97b]: https://github.com/germsvel/phoenix_test/commit/7a9b97b\n[891733f]: https://github.com/germsvel/phoenix_test/commit/891733f\n[b1400b3]: https://github.com/germsvel/phoenix_test/commit/b1400b3\n[515242d]: https://github.com/germsvel/phoenix_test/commit/515242d\n[8edd7b4]: https://github.com/germsvel/phoenix_test/commit/8edd7b4\n[6f610f2]: https://github.com/germsvel/phoenix_test/commit/6f610f2\n[2b73714]: https://github.com/germsvel/phoenix_test/commit/2b73714\n[2f6f3f6]: https://github.com/germsvel/phoenix_test/commit/2f6f3f6\n[6271e81]: https://github.com/germsvel/phoenix_test/commit/6271e81\n[45b62ca]: https://github.com/germsvel/phoenix_test/commit/45b62ca\n[ec4440d]: https://github.com/germsvel/phoenix_test/commit/ec4440d\n[04ca498]: https://github.com/germsvel/phoenix_test/commit/04ca498\n[ac52f8b]: https://github.com/germsvel/phoenix_test/commit/ac52f8b\n[a327ccf]: https://github.com/germsvel/phoenix_test/commit/a327ccf","ref":"changelog.html#fixes"},{"type":"extras","title":"0.6.0 - Changelog","doc":"","ref":"changelog.html#0-6-0"},{"type":"extras","title":"Breaking changes & deprecations - Changelog","doc":"- (potentially breaking) Raise error when visiting an defined route. Commit [668448a]\n- Deprecate select with `:from` in favor of `:option` (#191). Commit [071c3f3]\n\nFor a more detailed explanation, see the [Upgrade\nguides](https://hexdocs.pm/phoenix_test/upgrade_guides.html)","ref":"changelog.html#breaking-changes-deprecations"},{"type":"extras","title":"Improvements - Changelog","doc":"- Simplify timeout logic. Attempt faster checks. Commits [24b31e5], [a053b51],\n[d17baef], [1819f52], [1182dba]\n\n- Allow nesting of `within/3` (#189). Commit [94c716f]\n- Allow calling `visit/2` sequentially. Commit [230c6a1]\n- Include default email input fields in form data (#180). Commit [5e36c09]\n- Add mix benchmark alias to run benchmarks. Commit [0642aeb]\n- Support role=\"button\" for `click_button/2`. Commit [1dec249]\n- Document reason behind `fill_in` needing a label (#188). Commit [74732ab]\n- Docs: Mention playwright driver (#179). Commit [7619257]","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix documentation typo on  docs. Commit [756cc4c]\n- Throw nice error messages for `render_upload` errors (#177). Commit [a1cf133]\n\n[071c3f3]: https://github.com/germsvel/phoenix_test/commit/071c3f3\n[a053b51]: https://github.com/germsvel/phoenix_test/commit/a053b51\n[d17baef]: https://github.com/germsvel/phoenix_test/commit/d17baef\n[1dec249]: https://github.com/germsvel/phoenix_test/commit/1dec249\n[a4fb70a]: https://github.com/germsvel/phoenix_test/commit/a4fb70a\n[3c601c4]: https://github.com/germsvel/phoenix_test/commit/3c601c4\n[ce95e76]: https://github.com/germsvel/phoenix_test/commit/ce95e76\n[94c716f]: https://github.com/germsvel/phoenix_test/commit/94c716f\n[230c6a1]: https://github.com/germsvel/phoenix_test/commit/230c6a1\n[a1cf133]: https://github.com/germsvel/phoenix_test/commit/a1cf133\n[5e36c09]: https://github.com/germsvel/phoenix_test/commit/5e36c09\n[74732ab]: https://github.com/germsvel/phoenix_test/commit/74732ab\n[1819f52]: https://github.com/germsvel/phoenix_test/commit/1819f52\n[1182dba]: https://github.com/germsvel/phoenix_test/commit/1182dba\n[24b31e5]: https://github.com/germsvel/phoenix_test/commit/24b31e5\n[668448a]: https://github.com/germsvel/phoenix_test/commit/668448a\n[7619257]: https://github.com/germsvel/phoenix_test/commit/7619257\n[0642aeb]: https://github.com/germsvel/phoenix_test/commit/0642aeb\n[756cc4c]: https://github.com/germsvel/phoenix_test/commit/756cc4c","ref":"changelog.html#fixes"},{"type":"extras","title":"0.5.2 - Changelog","doc":"","ref":"changelog.html#0-5-2"},{"type":"extras","title":"Improvements - Changelog","doc":"- Allow wildcards in `assert_path` (#163). Commit [aec2a0d]\n- Add assertion benchmarks (#165). Commit [549cd49]\n\n- Improve `:timeout` implementation for assertion helpers.\n  - Improve LiveViewWatcher child spec. Commits [a3efdce], [31a5b79], [5da3bc6]\n  - Start LiveViewWatcher under the test supervisor. Commit [bbcaf35]\n  - Allow watcher to watch multiple views. Commit [0202671]\n  - Watch sequential LiveViews (via redirects) and async behavior. Commit\n    [f07c187]","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Catch exits when checking async pids. Commit [7f34790]\n- Use LiveView.Channel to fetch async pids. Commit [8721948]\n\n[5da3bc6]: https://github.com/germsvel/phoenix_test/commit/5da3bc6\n[e42543b]: https://github.com/germsvel/phoenix_test/commit/e42543b\n[9eeecad]: https://github.com/germsvel/phoenix_test/commit/9eeecad\n[549cd49]: https://github.com/germsvel/phoenix_test/commit/549cd49\n[8984387]: https://github.com/germsvel/phoenix_test/commit/8984387\n[c9f96c1]: https://github.com/germsvel/phoenix_test/commit/c9f96c1\n[ba5d999]: https://github.com/germsvel/phoenix_test/commit/ba5d999\n[8721948]: https://github.com/germsvel/phoenix_test/commit/8721948\n[f07c187]: https://github.com/germsvel/phoenix_test/commit/f07c187\n[0202671]: https://github.com/germsvel/phoenix_test/commit/0202671\n[aec2a0d]: https://github.com/germsvel/phoenix_test/commit/aec2a0d\n[7f34790]: https://github.com/germsvel/phoenix_test/commit/7f34790\n[89f31ff]: https://github.com/germsvel/phoenix_test/commit/89f31ff\n[a3efdce]: https://github.com/germsvel/phoenix_test/commit/a3efdce\n[31a5b79]: https://github.com/germsvel/phoenix_test/commit/31a5b79\n[bbcaf35]: https://github.com/germsvel/phoenix_test/commit/bbcaf35\n[6abace0]: https://github.com/germsvel/phoenix_test/commit/6abace0\n[3d78969]: https://github.com/germsvel/phoenix_test/commit/3d78969\n[1d420e6]: https://github.com/germsvel/phoenix_test/commit/1d420e6","ref":"changelog.html#fixes"},{"type":"extras","title":"Docs - Changelog","doc":"- Tag assertion docs. Commit [9eeecad]\n- Fix fill_in/4 doc example. Commit [c9f96c1]","ref":"changelog.html#docs"},{"type":"extras","title":"0.5.1 - Changelog","doc":"","ref":"changelog.html#0-5-1"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix race condition. Ping LiveView instead of using \"buffer\". Commit [3d78969]\n\n[3d78969]: https://github.com/germsvel/phoenix_test/commit/3d78969","ref":"changelog.html#fixes"},{"type":"extras","title":"0.5.0 - Changelog","doc":"","ref":"changelog.html#0-5-0"},{"type":"extras","title":"Added - Changelog","doc":"- Adds `assert_has/3` | `refute_has/3` `:timeout` option Commit [7cb6a41].\n- Support phx-trigger-action. Commit [f46ebc1].","ref":"changelog.html#added"},{"type":"extras","title":"Improvements - Changelog","doc":"- Allow LiveView 1.0. Commit [6e121c8].\n- Allow labels to have line breaks. Commit [5949951].\n- Make all functions part of the `Driver` protocol to allow for external\n  drivers. Commits [8c9e3be], [af5d011], [ef3fca0].\n- Raise error if implicit + explicit label found. Commit [9502fc7].","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Keep flash across Live -> Static redirects. Commit [ca7edf5].\n- Make sure live redirect copy headers. Commit [ce490d3].\n- Don't fail form when present disabled select. Commit [14b7bf0].\n- Update current path on static submit. Commit [46d30bb].\n\n[7cb6a41]: https://github.com/germsvel/phoenix_test/commit/7cb6a41\n[9775097]: https://github.com/germsvel/phoenix_test/commit/9775097\n[bfd1291]: https://github.com/germsvel/phoenix_test/commit/bfd1291\n[5cc4ea3]: https://github.com/germsvel/phoenix_test/commit/5cc4ea3\n[ca7edf5]: https://github.com/germsvel/phoenix_test/commit/ca7edf5\n[a2eaa24]: https://github.com/germsvel/phoenix_test/commit/a2eaa24\n[0af3b14]: https://github.com/germsvel/phoenix_test/commit/0af3b14\n[4000203]: https://github.com/germsvel/phoenix_test/commit/4000203\n[a346d09]: https://github.com/germsvel/phoenix_test/commit/a346d09\n[5949951]: https://github.com/germsvel/phoenix_test/commit/5949951\n[3506f7b]: https://github.com/germsvel/phoenix_test/commit/3506f7b\n[8c9e3be]: https://github.com/germsvel/phoenix_test/commit/8c9e3be\n[af5d011]: https://github.com/germsvel/phoenix_test/commit/af5d011\n[ef3fca0]: https://github.com/germsvel/phoenix_test/commit/ef3fca0\n[457144a]: https://github.com/germsvel/phoenix_test/commit/457144a\n[14b7bf0]: https://github.com/germsvel/phoenix_test/commit/14b7bf0\n[ce490d3]: https://github.com/germsvel/phoenix_test/commit/ce490d3\n[ddacde8]: https://github.com/germsvel/phoenix_test/commit/ddacde8\n[6e121c8]: https://github.com/germsvel/phoenix_test/commit/6e121c8\n[9502fc7]: https://github.com/germsvel/phoenix_test/commit/9502fc7\n[f46ebc1]: https://github.com/germsvel/phoenix_test/commit/f46ebc1\n[46d30bb]: https://github.com/germsvel/phoenix_test/commit/46d30bb\n[9eadbd9]: https://github.com/germsvel/phoenix_test/commit/9eadbd9\n[6d6d40b]: https://github.com/germsvel/phoenix_test/commit/6d6d40b","ref":"changelog.html#fixes"},{"type":"extras","title":"0.4.2 - Changelog","doc":"","ref":"changelog.html#0-4-2"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix: do not recycle fresh conn. Commit [24192e2]\n\n[24192e2]: https://github.com/germsvel/phoenix_test/commit/24192e2","ref":"changelog.html#fixes"},{"type":"extras","title":"0.4.1 - Changelog","doc":"","ref":"changelog.html#0-4-1"},{"type":"extras","title":"Added - Changelog","doc":"- Adds `select/4` `exact_option` option. Commit [f881da3]\n- Copy headers across redirects . Commit [7ad8cb1]","ref":"changelog.html#added"},{"type":"extras","title":"Fixes - Changelog","doc":"- 🛠️  Fix active form vs form data loading . Commit [37302b9]\n- Update `phx_click?` to check for valid JS commands. Commit [0d54b49]\n- Deduplicate data passed to FormPayload (multi checkbox fix). Commit [e950ae4]\n\n[0d54b49]: https://github.com/germsvel/phoenix_test/commit/0d54b49\n[e950ae4]: https://github.com/germsvel/phoenix_test/commit/e950ae4\n[7df460a]: https://github.com/germsvel/phoenix_test/commit/7df460a\n[06e43e9]: https://github.com/germsvel/phoenix_test/commit/06e43e9\n[ba34d9a]: https://github.com/germsvel/phoenix_test/commit/ba34d9a\n[b37199d]: https://github.com/germsvel/phoenix_test/commit/b37199d\n[3b175fb]: https://github.com/germsvel/phoenix_test/commit/3b175fb\n[73f2b83]: https://github.com/germsvel/phoenix_test/commit/73f2b83\n[37302b9]: https://github.com/germsvel/phoenix_test/commit/37302b9\n[7ad8cb1]: https://github.com/germsvel/phoenix_test/commit/7ad8cb1\n[5f106cd]: https://github.com/germsvel/phoenix_test/commit/5f106cd\n[f881da3]: https://github.com/germsvel/phoenix_test/commit/f881da3\n[5e05b4d]: https://github.com/germsvel/phoenix_test/commit/5e05b4d","ref":"changelog.html#fixes"},{"type":"extras","title":"0.4.0 - Changelog","doc":"","ref":"changelog.html#0-4-0"},{"type":"extras","title":"Breaking - Changelog","doc":"- Select options were previously matched inexactly (as a substring match). That\n  made it impossible to differentiate between two options where one was a subset\n  of the other.\n\n  For example, using `select(session, \"Email\", from: \"Contact\")` could not\n  differentiate between the `Email` and `Email and SMS` options. Select options\n  are now matched exactly. Commits [dc7ba01] and [e675561].\n\n  This is a technically a bug fix, but it's also a potentially breaking change\n  for existing tests that accidentally relied on that behavior. If you need\n  inexact matches on options, please open an issue describing your use case.\n\n[dc7ba01]: https://github.com/germsvel/phoenix_test/commit/dc7ba01\n[e675561]: https://github.com/germsvel/phoenix_test/commit/e675561","ref":"changelog.html#breaking"},{"type":"extras","title":"Added - Changelog","doc":"- Adds `PhoenixTest.upload/3` to test file uploads. Commit [d717970]\n- ✨ Form helpers now take `:exact` option. Commits [c4f9164] and [cb19f86]\n- Form helpers now allow passing CSS selectors to target inputs. Commits\n  [30b4eca], [12934b8], [bd595f8], [250e25e], [ef2999e]\n\n[d717970]: https://github.com/germsvel/phoenix_test/commit/d717970\n[c4f9164]: https://github.com/germsvel/phoenix_test/commit/c4f9164\n[cb19f86]: https://github.com/germsvel/phoenix_test/commit/cb19f86\n[30b4eca]: https://github.com/germsvel/phoenix_test/commit/30b4eca\n[12934b8]: https://github.com/germsvel/phoenix_test/commit/12934b8\n[bd595f8]: https://github.com/germsvel/phoenix_test/commit/bd595f8\n[250e25e]: https://github.com/germsvel/phoenix_test/commit/250e25e\n[ef2999e]: https://github.com/germsvel/phoenix_test/commit/ef2999e","ref":"changelog.html#added"},{"type":"extras","title":"Improvements - Changelog","doc":"- Raise nice error if field is missing `name` attribute. Commit [bb6950f]\n- Docs: Add syntax highlighting for heex and html. Commit [e308f9f]\n\n[bb6950f]: https://github.com/germsvel/phoenix_test/commit/bb6950f\n[e308f9f]: https://github.com/germsvel/phoenix_test/commit/e308f9f","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Ignore `phx-*` attrs when inferring selector. Commit [b5d28c8]\n- Don't error on default form input without name attribute. Commit [3bd6d7a]\n\n[b5d28c8]: https://github.com/germsvel/phoenix_test/commit/b5d28c8\n[3bd6d7a]: https://github.com/germsvel/phoenix_test/commit/3bd6d7a","ref":"changelog.html#fixes"},{"type":"extras","title":"0.3.2 - Changelog","doc":"","ref":"changelog.html#0-3-2"},{"type":"extras","title":"Added - Changelog","doc":"- Check/uncheck works with phx-click outside forms. Commit [82fa973]\n- Select works with phx-click outside forms. Commit [1493897]\n- Radio buttons work with phx-click outside forms. Commit [8745ee1]","ref":"changelog.html#added"},{"type":"extras","title":"Improvements - Changelog","doc":"- Send `_target` event with phx-change events. Commit [ce093f1]\n- Use `[id= ]` when querying by ID. Commits [5dd10f5] and [b79e8a3]\n- Allow non-string data types as inputs to fields. Commit [5cc0936]\n- Support default values on textareas. Commit [41c70e0]","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix `refute_has/2` to handle `:at` option without `text`. Commit [33a02bf]\n\n[82fa973]: https://github.com/germsvel/phoenix_test/commit/82fa973\n[1493897]: https://github.com/germsvel/phoenix_test/commit/1493897\n[8745ee1]: https://github.com/germsvel/phoenix_test/commit/8745ee1\n[ce093f1]: https://github.com/germsvel/phoenix_test/commit/ce093f1\n[5dd10f5]: https://github.com/germsvel/phoenix_test/commit/5dd10f5\n[b79e8a3]: https://github.com/germsvel/phoenix_test/commit/b79e8a3\n[5cc0936]: https://github.com/germsvel/phoenix_test/commit/5cc0936\n[41c70e0]: https://github.com/germsvel/phoenix_test/commit/41c70e0\n[33a02bf]: https://github.com/germsvel/phoenix_test/commit/33a02bf","ref":"changelog.html#fixes"},{"type":"extras","title":"0.3.1 - Changelog","doc":"","ref":"changelog.html#0-3-1"},{"type":"extras","title":"Improvements - Changelog","doc":"- Do not assume first option to be selected by default for multiple select. Commit [f146186]\n- Handle multiple select in forms. Commit [6b6512d]\n- Support pre-filled number inputs. Commit [81f03aa]","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix `open_browser/1` doc example. Commit [8f39959]\n- Deep merge form data in nested/prefixed forms. Commit [34b73b0]\n- Fix uncheck/2 in nested forms. Commit [e1a2408]\n- Fix: Preserve '?' in checkbox name. Commit [2e032ef]\n- Fix: Deep merge nested form button value (static). Commit [4303b4a]\n\n[8f39959]: https://github.com/germsvel/phoenix_test/commit/8f39959\n[34b73b0]: https://github.com/germsvel/phoenix_test/commit/34b73b0\n[f146186]: https://github.com/germsvel/phoenix_test/commit/f146186\n[81f03aa]: https://github.com/germsvel/phoenix_test/commit/81f03aa\n[e1a2408]: https://github.com/germsvel/phoenix_test/commit/e1a2408\n[2e032ef]: https://github.com/germsvel/phoenix_test/commit/2e032ef\n[4303b4a]: https://github.com/germsvel/phoenix_test/commit/4303b4a\n[6b6512d]: https://github.com/germsvel/phoenix_test/commit/6b6512d","ref":"changelog.html#fixes"},{"type":"extras","title":"0.3.0 - Changelog","doc":"","ref":"changelog.html#0-3-0"},{"type":"extras","title":"Added - Changelog","doc":"- Adds `PhoenixTest.unwrap/2` as an escape hatch. Commit [87be9c5].","ref":"changelog.html#added"},{"type":"extras","title":"Improvements - Changelog","doc":"- We now handle redirects on `phx-change` events. Commit [cf1687c].\n- Click button can submit forms when not nested in form. Commit [e173f5b].","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix `assert_path` live navigation with query params. Commit [6c58f27].","ref":"changelog.html#fixes"},{"type":"extras","title":"Removed - Changelog","doc":"- Removes deprecated `fill_form/3` and `submit_form/3`. Commit [fef7e82].\n- Removes deprecated `assert_has`/`refute_has` with text as positional argument.\n  Commit [82f4170].\n\n[87be9c5]: https://github.com/germsvel/phoenix_test/commit/87be9c5\n[cf1687c]: https://github.com/germsvel/phoenix_test/commit/cf1687c\n[e173f5b]: https://github.com/germsvel/phoenix_test/commit/e173f5b\n[6c58f27]: https://github.com/germsvel/phoenix_test/commit/6c58f27\n[fef7e82]: https://github.com/germsvel/phoenix_test/commit/fef7e82\n[82f4170]: https://github.com/germsvel/phoenix_test/commit/82f4170","ref":"changelog.html#removed"},{"type":"extras","title":"0.2.13 - Changelog","doc":"","ref":"changelog.html#0-2-13"},{"type":"extras","title":"Deprecations - Changelog","doc":"- Deprecates `fill_form/3` and `submit_form/3`. Commit [996458a]. See [upgrade\n  guides] for more info.\n\n[upgrade guides]: ./upgrade_guides.md#upgrading-to-0-2-13","ref":"changelog.html#deprecations"},{"type":"extras","title":"Additions - Changelog","doc":"- Adds `fill_in/3` helper. Commit [6c7a1d2].\n- Adds `select/3` helper. Commit [6efeadf].\n- Adds `choose/2` helper. Commit [5f58604].\n- Adds `within/3` helper. Commit [10b7269].\n- Adds `submit/1` helper. Commit [fff82d1].\n- Adds `check/2` and `uncheck/2` helpers. Commit [58110b4].\n\n[996458a]: https://github.com/germsvel/phoenix_test/commit/996458a\n[6c7a1d2]: https://github.com/germsvel/phoenix_test/commit/6c7a1d2\n[6efeadf]: https://github.com/germsvel/phoenix_test/commit/6efeadf\n[5f58604]: https://github.com/germsvel/phoenix_test/commit/5f58604\n[10b7269]: https://github.com/germsvel/phoenix_test/commit/10b7269\n[fff82d1]: https://github.com/germsvel/phoenix_test/commit/fff82d1\n[58110b4]: https://github.com/germsvel/phoenix_test/commit/58110b4","ref":"changelog.html#additions"},{"type":"extras","title":"0.2.12 - Changelog","doc":"","ref":"changelog.html#0-2-12"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix checked checkbox being overridden by hidden input. Commit [a621f34]\n- Handle multiple checkboxes inside a label. Commit [a8fc877]\n- Fix `assert_path`/`refute_path` to handle live patching. Commit [48a29a3]\n- Fix `assert_path`/`refute_path` to handle Live Navigation. Commit [842ab36]\n- Fix `assert_has`/`refute_has` title matching exactly. Commit [7b2f243]\n- Update `assert_has` examples in README to new API. Commit [cb7529b]\n\n[a621f34]: https://github.com/germsvel/phoenix_test/commit/a621f34\n[a8fc877]: https://github.com/germsvel/phoenix_test/commit/a8fc877\n[48a29a3]: https://github.com/germsvel/phoenix_test/commit/48a29a3\n[842ab36]: https://github.com/germsvel/phoenix_test/commit/842ab36\n[7b2f243]: https://github.com/germsvel/phoenix_test/commit/7b2f243\n[cb7529b]: https://github.com/germsvel/phoenix_test/commit/cb7529b","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.11 - Changelog","doc":"","ref":"changelog.html#0-2-11"},{"type":"extras","title":"Improvements - Changelog","doc":"- Add `assert_path` and `refute_path` assertions helpers. Commit [f2ab02c]\n- Include pre-selected text input values, selects, radio buttons, and checkboxes\n  in form submissions. Commits [32a8da5], [d253eba], [ebba679]\n- Include button's `name` and `value` if present. Commit [e54c64f]\n- `click_button` submits form without having to `fill_form` before it does.\n  Commit [8d16b7e]","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Follow multiple redirects in `Live.click_link`. Commit [28de102]\n\n[f2ab02c]: https://github.com/germsvel/phoenix_test/commit/f2ab02c\n[32a8da5]: https://github.com/germsvel/phoenix_test/commit/32a8da5\n[d253eba]: https://github.com/germsvel/phoenix_test/commit/d253eba\n[ebba679]: https://github.com/germsvel/phoenix_test/commit/ebba679\n[e54c64f]: https://github.com/germsvel/phoenix_test/commit/e54c64f\n[8d16b7e]: https://github.com/germsvel/phoenix_test/commit/8d16b7e\n[28de102]: https://github.com/germsvel/phoenix_test/commit/28de102","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.10 - Changelog","doc":"","ref":"changelog.html#0-2-10"},{"type":"extras","title":"Improvements - Changelog","doc":"- Add `count` option in assertions. Commit [16fd0a4]\n- Add `exact` option in assertions. Commit [da772f1]\n- Add `at` option in assertions. Commit [5da74ec]","ref":"changelog.html#improvements"},{"type":"extras","title":"Deprecations - Changelog","doc":"- Deprecate `assert_has/3` where `text` is the third argument (positional). Use\n  `assert_has/3` with `text:` option instead. Commit [193c21a]\n\n[16fd0a4]: https://github.com/germsvel/phoenix_test/commit/16fd0a4\n[da772f1]: https://github.com/germsvel/phoenix_test/commit/da772f1\n[5da74ec]: https://github.com/germsvel/phoenix_test/commit/5da74ec\n[193c21a]: https://github.com/germsvel/phoenix_test/commit/193c21a","ref":"changelog.html#deprecations"},{"type":"extras","title":"0.2.9 - Changelog","doc":"","ref":"changelog.html#0-2-9"},{"type":"extras","title":"Improvements - Changelog","doc":"- Adds `assert_has/2` and `refute_has/2`. Commits [3756a47] and [9709b7a].\n- Follows redirect on `visit/2`. Commit [b4f49be]","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Do not always assume `submit_button` after `fill_form` is for submitting the\n  form. Commit [e193a07]\n\n[3756a47]: https://github.com/germsvel/phoenix_test/commit/3756a47\n[9709b7a]: https://github.com/germsvel/phoenix_test/commit/9709b7a\n[b4f49be]: https://github.com/germsvel/phoenix_test/commit/b4f49be\n[e193a07]: https://github.com/germsvel/phoenix_test/commit/e193a07","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.8 - Changelog","doc":"","ref":"changelog.html#0-2-8"},{"type":"extras","title":"Added - Changelog","doc":"- Adds WSL2 for `open_browser/2`. Commit [b852014].\n- Relax Elixir version requirement to 1.15. Commit [3cb9586]\n- Support visiting non-200 pages in Static implementation. Commit [4964ab2].\n\n[b852014]: https://github.com/germsvel/phoenix_test/commit/b852014\n[3cb9586]: https://github.com/germsvel/phoenix_test/commit/3cb9586\n[4964ab2]: https://github.com/germsvel/phoenix_test/commit/4964ab2","ref":"changelog.html#added"},{"type":"extras","title":"0.2.7 - Changelog","doc":"","ref":"changelog.html#0-2-7"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fixes `open_browser/1` not existing. Commit [7407b19].\n\n[7407b19]: https://github.com/germsvel/phoenix_test/commit/7407b19","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.6 - Changelog","doc":"","ref":"changelog.html#0-2-6"},{"type":"extras","title":"Added - Changelog","doc":"- Adds `open_browser/1` function to both Live and Static implementations.\n  Commit [b9d8347].\n- Handle forms that use `data` attributes and `Phoenix.HTML.js` to submit forms.\n  Commit [d699792].","ref":"changelog.html#added"},{"type":"extras","title":"Fixes - Changelog","doc":"- Correctly handles forms that PUT/DELETE (through hidden inputs). Commit\n  [3efa4c0].\n\n[b9d8347]: https://github.com/germsvel/phoenix_test/commit/b9d8347\n[3efa4c0]: https://github.com/germsvel/phoenix_test/commit/3efa4c0\n[d699792]: https://github.com/germsvel/phoenix_test/commit/d699792","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.5 - Changelog","doc":"","ref":"changelog.html#0-2-5"},{"type":"extras","title":"Added - Changelog","doc":"- Introduce ability to assert and refute on page title. Commit [8552ec7].\n- Handle regular form submission from LiveView pages when using `submit_form`.\n  Commit [fc4d3ef].","ref":"changelog.html#added"},{"type":"extras","title":"Fixes - Changelog","doc":"- Improve Live validation of form fields to properly handle nested fields in\n  forms. Commits [180dc0d] and [c275c0c].\n\n[8552ec7]: https://github.com/germsvel/phoenix_test/commit/8552ec7\n[fc4d3ef]: https://github.com/germsvel/phoenix_test/commit/fc4d3ef\n[180dc0d]: https://github.com/germsvel/phoenix_test/commit/180dc0d\n[c275c0c]: https://github.com/germsvel/phoenix_test/commit/c275c0c","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.4 - Changelog","doc":"","ref":"changelog.html#0-2-4"},{"type":"extras","title":"Added - Changelog","doc":"- Handle form redirects from static pages. Commit\n  [4c39920](https://github.com/germsvel/phoenix_test/commit/4c39920)\n- Handle regular form submission from LiveView pages with `fill_form` +\n  `click_button`. Commit [fe755de](https://github.com/germsvel/phoenix_test/commit/fe755de)","ref":"changelog.html#added"},{"type":"extras","title":"Fixes - Changelog","doc":"- Use Html.raw/1 for more errors to handle nested buttons.\n  [82e7415](https://github.com/germsvel/phoenix_test/commit/82e7415)","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.3 - Changelog","doc":"","ref":"changelog.html#0-2-3"},{"type":"extras","title":"Added - Changelog","doc":"- Handle form redirects (to Live and static pages) from Live pages. Commit\n  [531e5e9](https://github.com/germsvel/phoenix_test/commit/531e5e9)\n- Expand documentation on nested forms. Commit\n  [6809389](https://github.com/germsvel/phoenix_test/commit/6809389)","ref":"changelog.html#added"},{"type":"extras","title":"Fixes - Changelog","doc":"- Allow multiple matching elements in `assert_has`. Commit\n  [ac0e167](https://github.com/germsvel/phoenix_test/commit/ac0e167)","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.2 - Changelog","doc":"","ref":"changelog.html#0-2-2"},{"type":"extras","title":"Added - Changelog","doc":"- Raise `AssertionError` instead of `RuntimeError` in assertions for more\n  consistent ExUnit error messages. Commit\n  [117bc59](https://github.com/germsvel/phoenix_test/commit/117bc59)\n- Update `fill_form` to handle that aren't direct children of the `form`\n  element. Commit\n  [46d6229](https://github.com/germsvel/phoenix_test/commit/46d6229)","ref":"changelog.html#added"},{"type":"extras","title":"0.2.1 - Changelog","doc":"","ref":"changelog.html#0-2-1"},{"type":"extras","title":"Added - Changelog","doc":"- Improve printing of complex nested content in assertions. Commit\n  [7151834](https://github.com/germsvel/phoenix_test/commit/7151834)\n- Better error messages in forms when multiple submit buttons/inputs are found.\n  Commit [82492c6](https://github.com/germsvel/phoenix_test/commit/82492c6)","ref":"changelog.html#added"},{"type":"extras","title":"Fixes - Changelog","doc":"- Allow using `refute_has` in pipes in the same way `assert_has` already works.\n  Commit [0484979](https://github.com/germsvel/phoenix_test/commit/0484979)","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.0 - Changelog","doc":"","ref":"changelog.html#0-2-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Update our static implementation to raise when we find many elements. That\n  brings it in line with how our LiveView implementation works. Commit\n  [daa4dca](https://github.com/germsvel/phoenix_test/commit/daa4dca)","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improved - Changelog","doc":"- Improves error messages in assertions. Commit [c995fc1](https://github.com/germsvel/phoenix_test/commit/c995fc1)","ref":"changelog.html#improved"},{"type":"extras","title":"0.1.1 - Changelog","doc":"","ref":"changelog.html#0-1-1"},{"type":"extras","title":"Added - Changelog","doc":"- Adds `click_link/3` and `click_button/3` which allow for specifying a CSS\n  selector. Commit [c7401b6](https://github.com/germsvel/phoenix_test/commit/c7401b6).","ref":"changelog.html#added"},{"type":"extras","title":"0.1.0 - Changelog","doc":"- Initial version of the library.","ref":"changelog.html#0-1-0"},{"type":"extras","title":"Upgrade Guides","doc":"Upgrade Guides\n==============","ref":"upgrade_guides.html"},{"type":"extras","title":"Upgrading to 0.7.0 - Upgrade Guides","doc":"Version 0.7.0 has a potentially breaking changes for those using the `upload`\nhelpers.\n\nPhoenixTest will now automatically trigger `phx-change` on `upload`.\n\nIf you were previously triggering `phx-change` after your `upload`, you might\nnow get duplicate `phx-change` events. Ideally, you simply no longer have to do\nthat.\n\nFor more information, see the discussion in PR [#162] or commit [8edd7b4].\n\n[#162]: https://github.com/germsvel/phoenix_test/pull/162\n[8edd7b4]: https://github.com/germsvel/phoenix_test/commit/8edd7b4","ref":"upgrade_guides.html#upgrading-to-0-7-0"},{"type":"extras","title":"Why the change? - Upgrade Guides","doc":"We always want PhoenixTest to behave as closely as possible like real Phoenix\ndoes. When you have an upload, it triggers your `phx-change` event. So, we want\nto emulate that. All other form helpers already do that. But `upload` didn't do\nit until now.","ref":"upgrade_guides.html#why-the-change"},{"type":"extras","title":"Upgrading to 0.6.0 - Upgrade Guides","doc":"Version 0.6.0 has one deprecation warning and one (potentially) breaking change.","ref":"upgrade_guides.html#upgrading-to-0-6-0"},{"type":"extras","title":"Deprecates `select/3` and `select/4` using `:from` (use `:option` instead) - Upgrade Guides","doc":"Deprecates `select/3` and `select/4` using `:from` to denote the label. Instead,\nit expects the label text to be passed as a positional argument and an `:option`\nkeyword argument to pass the option's text.\n\nThus, you'll need to make this change:\n\n```diff\n- |> select(\"Option 1\", from: \"Select Label\")\n+ |> select(\"Select Label\", option: \"Option 1\")\n```\n\nAnd if you're using the version that provides a CSS selector:\n\n```diff\n- |> select(\"#super-select\", \"Option 1\", from: \"Select Label\")\n+ |> select(\"#super-select\", \"Select Label\", option: \"Option 1\")\n```\n\n#### Why the change?\n\nIt may seem like a silly change (basically swapping positions of label and\noption arguments), and in some ways it is. There's no real change in\nfunctionality. So I've been very hesitant to make this change for a while.\n\nThe problem is that `select` is a bit surprising and confusing to use!\n\nAll other form helpers take in the label first as a positional argument, and\nthen any additional arguments (when they have them) go into a keyword list.\n\nBut `select` breaks that convention. It causes people to have to do mental\ngymnastics to switch the order of arguments.\n\nRather than live with confusion for the rest of our lives, it seems better to\nincur the cost right now, and then we can move on with all of our form helpers\nbeing consistent.","ref":"upgrade_guides.html#deprecates-select-3-and-select-4-using-from-use-option-instead"},{"type":"extras","title":"Raising when a route isn't found - Upgrade Guides","doc":"If you have any tests that navigate to a route that isn't defined in your\nrouter. Version 0.6.0 will raise an error.\n\nIf you visit a path that isn't defined, it's possible you were already getting\nan error -- in which case this isn't a breaking change for you.\n\nBut in some cases, people might've been landing on a page that wasn't defined\n(perhaps getting their 404 page), but their assertions still passed for other\nreasons (e.g. they were just asserting the path name)\n\nIn those cases, the change we introduced would be a breaking change.\n\n#### Why the change?\n\nWe want PhoenixTest to be as helpful as possible when you're test-driving the\nyour implementation. And we don't want it to provide false positives (meaning\nyour test passes, but it shouldn't pass).\n\nLet me give you two examples:\n\n1. Your test fails because the `assert_has` doesn't find the text on the page.\n   You know that text is on the page, so you're confused as to why that would\n   be. Only after you add an `open_browser` do you realize you had landed on a\n   404 or 500 page because there was a typo on a route somewhere. PhoenixTest\n   could just have raised an error that the route wasn't defined instead!\n\n2. You have a test that has a `refute_has` with some text. Your test passes, so\n   you think everything is good. Much later someone stumbles upon that test, but\n   when they use `open_browser`, they realize the `refute_has` was passing only\n   because you were on a 404 or 500 -- a completely different page from what you\n   thought! It turned out that you navigated to a path that didn't exist. You\n   thought your test was asserting that, for example, a user had been deleted,\n   but instead you had gone to a non-existent path.\n\nIn both of those cases, PhoenixTest could have saved you time and pain by simply\nensuring that the path you're trying to visit is a real one. And the sooner we\ncan give you feedback about that, the better.","ref":"upgrade_guides.html#raising-when-a-route-isn-t-found"},{"type":"extras","title":"Upgrading to 0.2.13 - Upgrade Guides","doc":"Version 0.2.13 deprecates `fill_form/3` and `submit_form/3`.\n\n🥺 I know it's a pain. I'm sorry about that.\n\nI don't take changing APIs lightly (even pre 1.0)... but I think you'll like\nthese changes.","ref":"upgrade_guides.html#upgrading-to-0-2-13"},{"type":"extras","title":"New form helpers - Upgrade Guides","doc":"Let me introduce you to our new form helpers:\n\n- `fill_in/3`\n- `select/3`\n- `choose/3`\n- `check/3`\n- `uncheck/3`\n\nThese new form helpers target elements by labels! 🥳\n\nInstead of relying on the underlying data structures generated by Phoenix\nforms and changesets, you can now specify which label you're targeting.\n\nChange this: 👇\n\n```elixir\nsession\n|> fill_form(\"form\", user: %{\n  name: \"Aragorn\",\n  admin: true,\n  country: \"Arnor\"\n})\n```\n\nTo this: 👇\n\n```elixir\nsession\n|> fill_in(\"Name\", with: \"Aragorn\")\n|> check(\"Admin\")\n|> select(\"Arnor\", from: \"Countries\")\n```\n\nThe new format:\n\n- encourages us (me included!) to use labels in forms,\n- decouples the testing of our forms from the underlying shape of a changeset or\n  Phoenix form -- something that's a mere implementation detail, and\n- allows us to express our tests closer to the language a user would use when\n  seeing a page.\n\n**But what if I don't want the label to show?**\n\nIt's a good idea to have labels for accessibility -- even if they're not visible\non the page. In those cases, you should hide them with CSS.\n\nFor example, if you use Tailwind, you can add a `sr-only` class to your label.\nThat will mark it as \"screen-reader only\" and hide it.","ref":"upgrade_guides.html#new-form-helpers"},{"type":"extras","title":"Targeting a form to fill out - Upgrade Guides","doc":"Since `fill_form/3` used to allow targeting a form by CSS selector, you may want\nto target a form via CSS selector with the new format. To do that, you can scope\nall of the form helpers using `within/3`:\n\n```elixir\nsession\n|> within(\"#user-form\", fn session ->\n  session\n  |> fill_in(\"Name\", with: \"Aragorn\")\n  |> check(\"Admin\")\n  |> select(\"Arnor\", from: \"Countries\")\nend)\n```\n\nNOTE: you may no longer _need_ to target your form via CSS selector. The new\nhelpers are a lot smarter since they're looking for the labels and their\nassociated inputs or options.\n\nBut if you have multiple forms with the same labels (even when those labels\npoint to different inputs), then you might have to scope your form-filling. And\nthat's where `within/3` can be handy.","ref":"upgrade_guides.html#targeting-a-form-to-fill-out"},{"type":"extras","title":"Submitting forms without clicking a button - Upgrade Guides","doc":"Once we've filled out a form, we typically click a button with `click_button/2`\nto submit the form. But sometimes you want to emulate what would happen by just\npressing   (or do what `submit_form/3` used to do).\n\nFor that case, you can use `submit/1` to submit the form you just filled out.\n\n```elixir\nsession\n|> fill_in(\"Name\", with: \"Aragorn\")\n|> check(\"Admin\")\n|> select(\"Arnor\", from: \"Countries\")\n|> submit()\n```","ref":"upgrade_guides.html#submitting-forms-without-clicking-a-button"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}